@@ -32,7 +32,7 @@ from calculate.lib.utils.portage import isPkgInstalled
 from calculate.lib.utils.device import getUdevDeviceInfo
 from crypt import crypt
 from autopartition import SchemeOpt, SchemeDevices
-from calculate.lib.encrypt import encrypt
+from calculate.lib.encrypt import encrypt, get_shadow_hash, get_grub_hash
 import calculate.lib.cl_ini_parser as cl_ini_parser
 
 from calculate.lib.cl_lang import setLocalTranslate, _
@@ -45,6 +45,7 @@ class UserHelper(VariableInterface):
     Locale variables not using for flash installation
     """
     xorg_need = False
+    stub_hash_value = "{SHA256}"
 
     def uncompatible(self):
         """
@@ -103,7 +104,7 @@ class VariableOsFormatUse(ReadonlyVariable):
         return map(self.checkFunc, self.Get('os_format_type'))
 
 
-class VariableClMigrateRootPwd(UserHelper, Variable):
+class VariableClMigrateRootPwdPlain(UserHelper, Variable):
     """
     Root password
     """
@@ -112,10 +113,39 @@ class VariableClMigrateRootPwd(UserHelper, Variable):
     metavalue = 'PASSWORD'
     untrusted = True
 
+    def get(self):
+        shadow_pwd = self.Get('cl_migrate_root_shadow_pwd')
+        if shadow_pwd:
+            return self.stub_hash_value
+        return ""
+
     def init(self):
         self.help = _("specify the root password")
         self.label = _("Root password")
 
+    def check(self, value):
+        if not value and not self.Get('cl_migrate_root_shadow_pwd'):
+            raise PasswordError(_("Password for user %s missing") % "root")
+
+
+class VariableClMigrateRootPwd(ReadonlyVariable):
+    """
+    Хэш root пароля
+    """
+    def get(self):
+        value = self.Get('cl_migrate_root_pwd_plain')
+
+        if value and value != UserHelper.stub_hash_value:
+            return get_shadow_hash().hash(value)
+
+        return self.Get('cl_migrate_root_shadow_pwd')
+
+
+class VariableClMigrateRootShadowPwd(ReadonlyVariable):
+    """
+    Хэш root пароля из файла /etc/shadow. Если пароль root, то
+    содержит пустую строку
+    """
     def get(self):
         rootPasswd = map(lambda x: x[1],
                          filter("root".__eq__,
@@ -127,26 +157,101 @@ class VariableClMigrateRootPwd(UserHelper, Variable):
             rootPasswd = ""
         # if root password is "root"
         if rootPasswd:
-            salt = "".join(rootPasswd.rpartition("$")[:1])
-            if salt and crypt("root", salt) == rootPasswd:
+            if get_shadow_hash().verify("root", rootPasswd):
                 rootPasswd = ""
         return rootPasswd or ""
 
+
+class GrubHelper(VariableInterface):
+    grub_passwd_file = "/etc/grub.d/07_passwd"
+
+    def read_hash_from_passwd(self):
+        """
+        Получить пароль из конфигурационного файла grub
+        """
+        data = readFile(self.grub_passwd_file)
+        reRootPwd = re.compile("password_pbkdf2 root (\S+)")
+        pwd = reRootPwd.search(data)
+        if pwd:
+            return pwd.group(1)
+        return ""
+
+
+class VariableClGrubPasswdSet(GrubHelper, Variable):
+    """
+    Использовать при установке системы пароль root как пароль для grub
+    """
+    type = 'bool'
+    opt = ["--grub-passwd"]
+
+    def init(self):
+        self.help = _("Use password for grub")
+        self.label = _("use password for grub")
+
+    def get(self):
+        if path.exists(self.grub_passwd_file):
+            return Variable.On
+        else:
+            return Variable.Off
+
+
+class VariableClGrubPwd(GrubHelper, Variable):
+    """
+    Хэш пароля на grub
+    """
+    opt = ["--passwd"]
+    type = "password"
+
+    def init(self):
+        self.help = _("grub password")
+        self.label = _("Grub password")
+
+    def get(self):
+        if self.GetBool('cl_grub_remove_pwd_set'):
+            return ""
+        passwd_hash = self.read_hash_from_passwd()
+        if passwd_hash:
+            return passwd_hash
+        if self.GetBool('cl_grub_passwd_set'):
+            value = self.Get('cl_migrate_root_pwd_plain')
+            if value and value != UserHelper.stub_hash_value:
+                enc = get_grub_hash()
+                return enc.hash(value)
+        return ""
+
     def set(self, value):
         """
-        Encrypt password
+        Поддержка использвания как шифрованного хэша так и plain
         """
-        reCheck = re.compile("^\$[^$]+\$[^$]+\$.*$")
-        encryptObj = encrypt()
-
-        if reCheck.match(value) or not value:
+        enc = get_grub_hash()
+        # используется hash
+        if enc.identify(value):
             return value
-        else:
-            return encryptObj.getHashPasswd(value, "shadow_ssha256")
+        # отключение
+        if value in ("", "none"):
+            return ""
+        # используется plain
+        return enc.hash(value)
 
     def check(self, value):
-        if not value:
-            raise PasswordError(_("Password for user %s missing") % "root")
+        if value and self.GetBool('cl_grub_remove_pwd_set'):
+            raise VariableError(
+                _("Setting the password is uncompatible with removing the password"))
+
+
+class VariableClGrubRemovePwdSet(Variable):
+    """
+    Удалить пароль из grub
+    """
+    type = "bool"
+    guitype = "hidden"
+    value = Variable.Off
+
+    opt = ["--remove-passwd"]
+
+    def init(self):
+        self.help = _("remove grub password")
+        self.label = _("Remove grub password")
 
 
 class VariableClInstallHomeCryptSet(UserHelper, Variable):
@@ -303,12 +408,10 @@ class VariableClMigrateUserPwd(UserHelper, Variable):
         """
         Encrypt passwords
         """
-        reCheck = re.compile("^\$[^$]+\$[^$]+\$.*$")
-        encryptObj = encrypt()
+        shadow_hash = get_shadow_hash()
 
-        return map(lambda x: x if reCheck.match(x) or not x else \
-            encryptObj.getHashPasswd(x, "shadow_ssha256"),
-                   value)
+        return map(lambda x: x if shadow_hash.identify(x) or not x else \
+            shadow_hash.hash(x), value)
 
 
 class VariableClAutologin(UserHelper, Variable):
