@@ -109,7 +109,9 @@ class Wsdl(WsdlBase):
                                         'os_install_net_route_data')),
             lambda group: group(_("Users"),
                                 normal=(
-                                    'cl_migrate_root_pwd', 'cl_migrate_data',
+                                    'cl_migrate_root_pwd_plain',
+                                    'cl_grub_passwd_set',
+                                    'cl_migrate_data',
                                     'cl_autologin'),
                                 expert=('cl_install_home_crypt_set',),
                                 hide=('cl_migrate_data',),
@@ -208,10 +210,13 @@ class Wsdl(WsdlBase):
             'groups': [
                 lambda group: group(_("Boot"),
                                     normal=(
-                                        'os_install_mbr', 'os_install_uefi_set',
+                                        'os_install_mbr',
+                                        'os_install_uefi_set',
                                         'os_install_kernel_scheduler',
+                                        'cl_grub_remove_pwd_set',
                                         'os_install_grub_terminal'),
                                     expert=(
+                                        'cl_grub_pwd',
                                         'cl_templates_locate',
                                         'cl_dispatch_conf',
                                         'cl_verbose_set'),
